=== modified file 'bzrlib/changeset.py'
--- bzrlib/changeset.py	2005-10-14 03:50:50 +0000
+++ bzrlib/changeset.py	2005-10-14 04:32:24 +0000
@@ -944,8 +944,7 @@
                 except OSError, e:
                     if e.errno != errno.ENOENT:
                         raise
-                    if conflict_handler.missing_for_rename(src_path, to_name) \
-                        == "skip":
+                    if conflict_handler.missing_for_rename(src_path) == "skip":
                         continue
 
     return temp_name
@@ -967,7 +966,6 @@
     :type reverse: bool
     """
     for entry in target_entries:
-        print entry.contents_change
         new_tree_path = entry.get_new_path(inventory, changeset, reverse)
         if new_tree_path is None:
             continue
@@ -1073,8 +1071,8 @@
 
 
 class MissingForRename(Exception):
-    def __init__(self, filename, to_path):
-        msg = "Attempt to move missing path %s to %s" % (filename, to_path)
+    def __init__(self, filename):
+        msg = "Attempt to move missing path %s" % (filename)
         Exception.__init__(self, msg)
         self.filename = filename
 
@@ -1150,8 +1148,8 @@
     def missing_for_rm(self, filename, change):
         raise MissingForRm(filename)
 
-    def missing_for_rename(self, filename, to_path):
-        raise MissingForRename(filename, to_path)
+    def missing_for_rename(self, filename):
+        raise MissingForRename(filename)
 
     def missing_for_merge(self, file_id, other_path):
         raise MissingForMerge(other_path)
@@ -1224,7 +1222,7 @@
     r_inventory = {}
     for entry in tree.source_inventory().itervalues():
         inventory[entry.id] = entry.path
-    new_inventory = apply_changeset(cset, r_inventory, tree.root,
+    new_inventory = apply_changeset(cset, r_inventory, tree.basedir,
                                     reverse=reverse)
     new_entries, remove_entries = \
         get_inventory_change(inventory, new_inventory, cset, reverse)
@@ -1429,9 +1427,9 @@
             yield self.get_entry(file_id, tree)
 
     def get_entry(self, file_id, tree):
-        if not tree.has_id(file_id, allow_root=True):
+        if not tree.has_or_had_id(file_id):
             return None
-        return tree.inventory[file_id]
+        return tree.tree.inventory[file_id]
 
     def get_entry_parent(self, entry):
         if entry is None:
@@ -1439,7 +1437,7 @@
         return entry.parent_id
 
     def get_path(self, file_id, tree):
-        if not tree.has_id(file_id, allow_root=True):
+        if not tree.has_or_had_id(file_id):
             return None
         path = tree.id2path(file_id)
         if path == '':
@@ -1535,7 +1533,7 @@
 
 
 def full_path(entry, tree):
-    return os.path.join(tree.root, entry.path)
+    return os.path.join(tree.basedir, entry.path)
 
 def new_delete_entry(entry, tree, inventory, delete):
     if entry.path == "":

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-10-14 03:50:50 +0000
+++ bzrlib/merge.py	2005-10-14 04:32:24 +0000
@@ -154,11 +154,11 @@
     def abs_this_path(self, file_id):
         """Return the absolute path for a file_id in the this tree."""
         relpath = self.this_tree.id2path(file_id)
-        return self.this_tree.abspath(relpath)
+        return self.this_tree.tree.abspath(relpath)
 
     def add_missing_parents(self, file_id, tree):
         """If some of the parents for file_id are missing, add them."""
-        entry = tree.inventory[file_id]
+        entry = tree.tree.inventory[file_id]
         if entry.parent_id not in self.this_tree:
             return self.create_all_missing(entry.parent_id, tree)
         else:
@@ -166,7 +166,7 @@
 
     def create_all_missing(self, file_id, tree):
         """Add contents for a file_id and all its parents to a tree."""
-        entry = tree.inventory[file_id]
+        entry = tree.tree.inventory[file_id]
         if entry.parent_id is not None and entry.parent_id not in self.this_tree:
             abspath = self.create_all_missing(entry.parent_id, tree)
         else:
@@ -198,7 +198,7 @@
         if not self.ignore_zero:
             note("%d conflicts encountered.\n" % self.conflicts)
             
-def get_tree(treespec, local_branch=None):
+def get_tree(treespec, temp_root, label, local_branch=None):
     location, revno = treespec
     branch = Branch.open_containing(location)
     if revno is None:
@@ -207,17 +207,21 @@
         revision = branch.last_revision()
     else:
         revision = branch.get_rev_id(revno)
-    return branch, get_revid_tree(branch, revision, local_branch)
+    return branch, get_revid_tree(branch, revision, temp_root, label,
+                                  local_branch)
 
-def get_revid_tree(branch, revision, local_branch):
+def get_revid_tree(branch, revision, temp_root, label, local_branch):
     if revision is None:
-        return branch.working_tree()
+        base_tree = branch.working_tree()
     else:
         if local_branch is not None:
             greedy_fetch(local_branch, branch, revision)
-            return local_branch.revision_tree(revision)
+            base_tree = local_branch.revision_tree(revision)
         else:
-            return branch.revision_tree(revision)
+            base_tree = branch.revision_tree(revision)
+    temp_path = os.path.join(temp_root, label)
+    os.mkdir(temp_path)
+    return MergeAdapterTree(base_tree, temp_path)
 
 
 def file_exists(tree, file_id):
@@ -234,9 +238,9 @@
     def __init__(self, tree, tempdir):
         object.__init__(self)
         if hasattr(tree, "basedir"):
-            self.root = tree.basedir
+            self.basedir = tree.basedir
         else:
-            self.root = None
+            self.basedir = None
         self.tree = tree
         self.tempdir = tempdir
         os.mkdir(os.path.join(self.tempdir, "texts"))
@@ -265,9 +269,7 @@
         return self.tree.has_id(file_id)
 
     def has_or_had_id(self, file_id):
-        if file_id == self.tree.inventory.root.file_id:
-            return True
-        return self.tree.inventory.has_id(file_id)
+        return self.tree.has_or_had_id(file_id)
 
     def kind(self, file_id):
         return self.tree.kind(file_id)
@@ -328,7 +330,8 @@
                                     this_branch.basis_tree(), False)
             if changes.has_changed():
                 raise BzrCommandError("Working tree has uncommitted changes.")
-        other_branch, other_tree = get_tree(other_revision, this_branch)
+        other_branch, other_tree = get_tree(other_revision, tempdir, "other",
+                                            this_branch)
         if other_revision[1] == -1:
             other_rev_id = other_branch.last_revision()
             if other_rev_id is None:
@@ -348,10 +351,11 @@
                                               this_branch)
             except NoCommonAncestor:
                 raise UnrelatedBranches()
-            base_tree = get_revid_tree(this_branch, base_rev_id, None)
+            base_tree = get_revid_tree(this_branch, base_rev_id, tempdir, 
+                                       "base", None)
             base_is_ancestor = True
         else:
-            base_branch, base_tree = get_tree(base_revision)
+            base_branch, base_tree = get_tree(base_revision, tempdir, "base")
             if base_revision[1] == -1:
                 base_rev_id = base_branch.last_revision()
             elif base_revision[1] is None:
@@ -369,7 +373,7 @@
             for fname in file_list:
                 path = this_branch.relpath(fname)
                 found_id = False
-                for tree in (this_tree, base_tree, other_tree):
+                for tree in (this_tree, base_tree.tree, other_tree.tree):
                     file_id = tree.inventory.path2id(path)
                     if file_id is not None:
                         interesting_ids.add(file_id)
@@ -405,10 +409,10 @@
             contents_change = BackupBeforeChange(contents_change)
         return contents_change
 
-    this_tree = get_tree((this_branch.base, None))[1]
+    this_tree = get_tree((this_branch.base, None), tempdir, "this")[1]
 
     def get_inventory(tree):
-        return tree.inventory
+        return tree.tree.inventory
 
     inv_changes = merge_flex(this_tree, base_tree, other_tree,
                              generate_changeset, get_inventory,
@@ -427,7 +431,7 @@
             path = path[2:]
         adjust_ids.append((path, id))
     if len(adjust_ids) > 0:
-        this_branch.set_inventory(regen_inventory(this_branch,
+        this_branch.set_inventory(regen_inventory(this_branch, 
                                                   this_tree.basedir,
                                                   adjust_ids))
 

=== modified file 'bzrlib/merge_core.py'
--- bzrlib/merge_core.py	2005-10-14 03:50:50 +0000
+++ bzrlib/merge_core.py	2005-10-14 04:32:24 +0000
@@ -107,7 +107,7 @@
     cset = changeset_function(base, other, interesting_ids)
     new_cset = make_merge_changeset(cset, this, base, other, 
                                     conflict_handler, merge_factory)
-    result = apply_changeset(new_cset, invert_invent(this.inventory),
+    result = apply_changeset(new_cset, invert_invent(this.tree.inventory),
                              this.basedir, conflict_handler, False)
     conflict_handler.finalize()
     return result
@@ -127,7 +127,6 @@
             new_contents = make_merged_contents(entry, this, base, other, 
                                                 conflict_handler,
                                                 merge_factory)
-            print new_contents.is_creation()
             new_entry.contents_change = new_contents
             new_entry.metadata_change = make_merged_metadata(entry, base, other)
             new_cset.add_entry(new_entry)
@@ -159,9 +158,9 @@
     from bzrlib.trace import mutter
     def entry_data(file_id, tree):
         assert hasattr(tree, "__contains__"), "%s" % tree
-        if not tree.has_id(file_id, allow_root=True):
+        if not tree.has_or_had_id(file_id):
             return (None, None, "")
-        entry = tree.inventory[file_id]
+        entry = tree.tree.inventory[file_id]
         my_dir = tree.id2path(entry.parent_id)
         if my_dir is None:
             my_dir = ""

=== modified file 'bzrlib/selftest/test_merge_core.py'
--- bzrlib/selftest/test_merge_core.py	2005-10-14 03:50:50 +0000
+++ bzrlib/selftest/test_merge_core.py	2005-10-14 04:32:24 +0000
@@ -29,14 +29,14 @@
         return entry
         
     def make_inventory_entry(self, file_id):
-        path = self._realtree.inventory_dict.get(file_id)
+        path = self._realtree.inventory.get(file_id)
         if path is None:
             return None
         if path == "":
             return RootEntry(file_id)
         dir, name = os.path.split(path)
         kind = file_kind(self._realtree.abs_path(path))
-        for parent_id, path in self._realtree.inventory_dict.iteritems():
+        for parent_id, path in self._realtree.inventory.iteritems():
             if path == dir:
                 break
         if path != dir:
@@ -55,11 +55,11 @@
     def __init__(self, dir):
         self.dir = dir;
         os.mkdir(dir)
-        self.inventory_dict = {'0': ""}
-        self.inventory = FalseTree(self)
+        self.inventory = {'0': ""}
+        self.tree = FalseTree(self)
     
     def child_path(self, parent, name):
-        return os.path.join(self.inventory_dict[parent], name)
+        return os.path.join(self.inventory[parent], name)
 
     def add_file(self, id, parent, name, contents, mode):
         path = self.child_path(parent, name)
@@ -67,18 +67,18 @@
         assert not os.path.exists(full_path)
         file(full_path, "wb").write(contents)
         os.chmod(self.abs_path(path), mode)
-        self.inventory_dict[id] = path
+        self.inventory[id] = path
 
     def add_symlink(self, id, parent, name, target):
         path = self.child_path(parent, name)
         full_path = self.abs_path(path)
         assert not os.path.exists(full_path)
         os.symlink(target, full_path)
-        self.inventory_dict[id] = path
+        self.inventory[id] = path
 
     def remove_file(self, id):
         os.unlink(self.full_path(id))
-        del self.inventory_dict[id]
+        del self.inventory[id]
 
     def add_dir(self, id, parent, name, mode):
         path = self.child_path(parent, name)
@@ -86,14 +86,14 @@
         assert not os.path.exists(full_path)
         os.mkdir(self.abs_path(path))
         os.chmod(self.abs_path(path), mode)
-        self.inventory_dict[id] = path
+        self.inventory[id] = path
 
     def abs_path(self, path):
         return os.path.join(self.dir, path)
 
     def full_path(self, id):
         try:
-            tree_path = self.inventory_dict[id]
+            tree_path = self.inventory[id]
         except KeyError:
             return None
         return self.abs_path(tree_path)
@@ -102,10 +102,9 @@
         return self.full_path(id)
 
     def __contains__(self, file_id):
-        return file_id in self.inventory_dict
+        return file_id in self.inventory
 
-    def has_id(self, file_id, allow_root=False):
-        assert allow_root == True
+    def has_or_had_id(self, file_id):
         return file_id in self
 
     def get_file(self, file_id):
@@ -113,15 +112,15 @@
         return file(path, "rb")
 
     def id2path(self, file_id):
-        return self.inventory_dict[file_id]
+        return self.inventory[file_id]
 
     def id2abspath(self, id):
         return self.full_path(id)
 
     def change_path(self, id, path):
-        old_path = os.path.join(self.dir, self.inventory_dict[id])
+        old_path = os.path.join(self.dir, self.inventory[id])
         rename(old_path, self.abs_path(path))
-        self.inventory_dict[id] = path
+        self.inventory[id] = path
 
     def is_executable(self, file_id):
         mode = os.lstat(self.full_path(file_id)).st_mode
@@ -341,11 +340,11 @@
 
     def apply_changeset(self, cset, conflict_handler=None, reverse=False):
         inventory_change = changeset.apply_changeset(cset,
-                                                     self.this.inventory_dict,
+                                                     self.this.inventory,
                                                      self.this.dir,
                                                      conflict_handler, reverse)
-        self.this.inventory_dict =  self.apply_inv_change(inventory_change, 
-                                                     self.this.inventory_dict)
+        self.this.inventory =  self.apply_inv_change(inventory_change, 
+                                                     self.this.inventory)
 
     def cleanup(self):
         shutil.rmtree(self.dir)
@@ -369,7 +368,7 @@
         for tree in (builder.this, builder.other, builder.base):
             assert(tree.dir != builder.dir and 
                    tree.dir.startswith(builder.dir))
-            for path in tree.inventory_dict.itervalues():
+            for path in tree.inventory.itervalues():
                 fullpath = tree.abs_path(path)
                 assert(fullpath.startswith(tree.dir))
                 assert(not path.startswith(tree.dir))
@@ -392,11 +391,11 @@
         builder.add_file("4", "1", "file2", "hello2", 0644)
         builder.add_file("5", "1", "file3", "hello3", 0644)
         builder.change_parent("3", other="2")
-        assert(Inventory(builder.other.inventory_dict).get_parent("3") == "2")
+        assert(Inventory(builder.other.inventory).get_parent("3") == "2")
         builder.change_parent("4", this="2")
-        assert(Inventory(builder.this.inventory_dict).get_parent("4") == "2")
+        assert(Inventory(builder.this.inventory).get_parent("4") == "2")
         builder.change_parent("5", base="2")
-        assert(Inventory(builder.base.inventory_dict).get_parent("5") == "2")
+        assert(Inventory(builder.base.inventory).get_parent("5") == "2")
         cset = builder.merge_changeset(ApplyMerge3)
         for id in ("1", "2", "4", "5"):
             assert(cset.entries[id].is_boring())

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2005-10-14 03:50:50 +0000
+++ bzrlib/tree.py	2005-10-14 04:32:24 +0000
@@ -55,8 +55,11 @@
         """True if the tree has given filename."""
         raise NotImplementedError()
 
-    def has_id(self, file_id, allow_root=False):
-        if allow_root and file_id == self.inventory.root.file_id:
+    def has_id(self, file_id):
+        return self.inventory.has_id(file_id)
+
+    def has_or_had_id(self, file_id):
+        if file_id == self.inventory.root.file_id:
             return True
         return self.inventory.has_id(file_id)
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-10-14 03:50:50 +0000
+++ bzrlib/workingtree.py	2005-10-14 04:32:24 +0000
@@ -156,16 +156,18 @@
         return self.abspath(self.id2path(file_id))
 
                 
-    def has_id(self, file_id, allow_root=False):
+    def has_id(self, file_id):
         # files that have been deleted are excluded
-        inv = self.inventory
-        if allow_root and file_id == inv.root.file_id:
-            return True
+        inv = self._inventory
         if not inv.has_id(file_id):
             return False
         path = inv.id2path(file_id)
         return bzrlib.osutils.lexists(self.abspath(path))
 
+    def has_or_had_id(self, file_id):
+        if file_id == self.inventory.root.file_id:
+            return True
+        return self.inventory.has_id(file_id)
 
     __contains__ = has_id
     

